<?php

namespace Drupal\self_evaluation;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Url;
use Drupal\self_evaluation\Entity\SelfEvaluation;
use Drupal\self_evaluation\Entity\SelfEvaluationQuestion;
use Drupal\self_evaluation\Entity\SelfEvaluationTheme;
use Drupal\self_evaluation\Entity\SelfEvaluationThemeThreshold;

/**
 * Trait to build Active Trail.
 */
trait ActiveTrailTrait {

  /**
   * Return the current Active Trail for an entity.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The current entity.
   *
   * @return array
   *   Array return.
   */
  public function buildActiveTrail(EntityInterface $entity): array {
    $build = [
      '#type' => 'html_tag',
      '#tag' => 'div',
      '#attributes' => ['class' => ['active-trail']],
    ];
    if ($entity instanceof SelfEvaluation) {
      $build['trail'] = $this->buildSelfEvaluationActiveTrail($entity);
    }
    if ($entity instanceof SelfEvaluationTheme) {
      $build['trail'] = $this->buildThemeActiveTrail($entity);
    }
    if ($entity instanceof SelfEvaluationThemeThreshold) {
      $build['trail'] = $this->buildThresholdActiveTrail($entity);
    }
    if ($entity instanceof SelfEvaluationQuestion) {
      $build['trail'] = $this->buildQuestionActiveTrail($entity);
    }
    return $build;
  }

  /**
   * Build the current active trail from a self evaluation route.
   *
   * @param \Drupal\self_evaluation\SelfEvaluationInterface $self_evaluation
   *   The current self evaluation.
   *
   * @return array
   *   Array Return.
   */
  public function buildSelfEvaluationActiveTrail(SelfEvaluationInterface $self_evaluation): array {
    return [
      '#type' => 'html_tag',
      '#tag' => 'span',
      '#value' => $this->t('Self Evaluation'),
      'points' => [
        '#type' => 'html_tag',
        '#tag' => 'span',
        '#value' => ' : ',
      ],
      'link' => [
        '#type' => 'link',
        '#title' => $self_evaluation->label(),
        '#url' => Url::fromRoute('entity.self_evaluation.edit_form', ['self_evaluation' => $self_evaluation->id()]),
      ],
    ];
  }

  /**
   * Build the current active trail from a theme route.
   *
   * @param \Drupal\self_evaluation\SelfEvaluationThemeInterface $theme
   *   The current theme.
   *
   * @return array
   *   Array Return.
   */
  public function buildThemeActiveTrail(SelfEvaluationThemeInterface $theme): array {
    return [
      'parent' => $this->buildSelfEvaluationActiveTrail($theme->self_evaluation->entity),
      'theme' => [
        '#type' => 'html_tag',
        '#tag' => 'span',
        '#value' => $this->t('Theme'),
        'points' => [
          '#type' => 'html_tag',
          '#tag' => 'span',
          '#value' => ' : ',
        ],
        'link' => [
          '#type' => 'link',
          '#title' => $theme->label(),
          '#url' => Url::fromRoute('entity.self_evaluation.themes', ['self_evaluation' => $theme->self_evaluation->entity->id()]),
        ],
      ],
    ];
  }

  /**
   * Build the current active trail from a theme route.
   *
   * @param \Drupal\self_evaluation\SelfEvaluationThemeThresholdInterface $threshold
   *   The current theme threshold.
   *
   * @return array
   *   Array Return.
   */
  public function buildThresholdActiveTrail(SelfEvaluationThemeThresholdInterface $threshold): array {
    return [
      'parent' => $this->buildSelfEvaluationActiveTrail($threshold->self_evaluation->entity),
      'threshold' => [
        '#type' => 'html_tag',
        '#tag' => 'span',
        '#value' => $this->t('Threshold'),
        'points' => [
          '#type' => 'html_tag',
          '#tag' => 'span',
          '#value' => ' : ',
        ],
        'link' => [
          '#type' => 'link',
          '#title' => $threshold->label(),
          '#url' => Url::fromRoute('entity.self_evaluation.thresholds', ['self_evaluation' => $threshold->self_evaluation->entity->id()]),
        ],
      ],
    ];
  }

  /**
   * Build the current active trail from a question route.
   *
   * @param \Drupal\self_evaluation\SelfEvaluationQuestionInterface $question
   *   The current question.
   *
   * @return array
   *   Array Return.
   */
  public function buildQuestionActiveTrail(SelfEvaluationQuestionInterface $question): array {
    return [
      'parent' => $this->buildThemeActiveTrail($question->self_evaluation_theme->entity),
      'theme' => [
        '#type' => 'html_tag',
        '#tag' => 'span',
        '#value' => $this->t('Question'),
        'points' => [
          '#type' => 'html_tag',
          '#tag' => 'span',
          '#value' => ' : ',
        ],
        'link' => [
          '#type' => 'link',
          '#title' => $question->label(),
          '#url' => Url::fromRoute('entity.self_evaluation_theme.questions', ['self_evaluation_theme' => $question->self_evaluation_theme->entity->id()]),
        ],
      ],
    ];
  }

}
