<?php

namespace Drupal\self_evaluation\Controller;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Url;
use Drupal\self_evaluation\ActiveTrailTrait;
use Drupal\self_evaluation\Entity\SelfEvaluationQuestion;
use Drupal\self_evaluation\Entity\SelfEvaluationQuestionChoice;
use Drupal\self_evaluation\SelfEvaluationThemeInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Returns responses for questions admin routes.
 */
class SelfEvaluationQuestionsController extends SelfEvaluationListController {

  use ActiveTrailTrait;

  /**
   * Constructs a new SelfEvaluationAdminController.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   Entity Type manager service.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;

  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): self {
    return new static(
      $container->get('entity_type.manager'),
    );
  }

  /**
   * Return questions List for a theme.
   *
   * @param \Drupal\self_evaluation\SelfEvaluationThemeInterface $self_evaluation_theme
   *   The current Theme Entity.
   *
   * @return array
   *   Array return.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function buildList(SelfEvaluationThemeInterface $self_evaluation_theme): array {
    $rows = [];
    $list_questions = SelfEvaluationQuestion::getQuestionsFromTheme($self_evaluation_theme);

    if (!empty($list_questions)) {
      $themes = $this->entityTypeManager->getStorage('self_evaluation_question')
        ->loadMultiple($list_questions);

      foreach ($themes as $theme) {
        $rows[] = $this->buildRow($theme);
      }
    }
    $build = $this->render();

    $build['build_active_trail'] = $this->buildActiveTrail($self_evaluation_theme);
    $build['add']['#title'] = $this->t('Add a question');
    $build['add']['#url'] = Url::fromRoute('entity.self_evaluation_question.add_form', [], ['query' => ['self_evaluation_theme' => $self_evaluation_theme->id()]]);
    $build['list']['#empty'] = $this->t('There are no question yet.');
    $build['list']['#rows'] = $rows;
    return $build;
  }

  /**
   * Builds the header row for the entity listing.
   *
   * @return array
   *   A render array structure of header strings.
   *
   * @see \Drupal\Core\Entity\EntityListBuilder::render()
   */
  public function buildHeader(): array {
    $header['title'] = $this->t('Title');
    $header['choices'] = $this->t('Choices');
    $header['weight'] = $this->t('Weight');
    $header['status'] = $this->t('Publishing status');
    return $header + parent::buildHeader();
  }

  /**
   * Builds a row for an entity in the entity listing.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity for this row of the list.
   *
   * @return array
   *   A render array structure of fields for this entity.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   *
   * @see \Drupal\Core\Entity\EntityListBuilder::render()
   */
  public function buildRow(EntityInterface $entity): array {
    $row['title']['data'] = $entity->label();
    $row['choices']['data'] = $this->getChoices($entity);
    $row['weight']['data'] = $entity->weight->value;
    $row['status']['data'] = $entity->isEnabled() ? $this->t('Published') : $this->t('Unpublished');
    return $row + parent::buildRow($entity);
  }

  /**
   * Get data for choices cell for a question.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   Current Entity Question.
   *
   * @return array
   *   Data.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function getChoices(EntityInterface $entity): array {
    $list_choices = SelfEvaluationQuestionChoice::getChoicesForQuestion($entity);
    if (count($list_choices) > 0) {
      return [
        '#type' => 'link',
        '#title' => $this->t("List of choices"),
        '#url' => Url::fromRoute('entity.self_evaluation_question.choices', ['self_evaluation_question' => $entity->id()]),
      ];
    }
    return [
      '#type' => 'link',
      '#title' => $this->t("Add a choice"),
      '#url' => Url::fromRoute('entity.self_evaluation_question_choice.add_form', [], ['query' => ['self_evaluation_question' => $entity->id()]]),
    ];
  }

}
