<?php

namespace Drupal\self_evaluation\Controller;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Url;
use Drupal\self_evaluation\ActiveTrailTrait;
use Drupal\self_evaluation\Entity\SelfEvaluationQuestion;
use Drupal\self_evaluation\Entity\SelfEvaluationTheme;
use Drupal\self_evaluation\Entity\SelfEvaluationThemeThreshold;
use Drupal\self_evaluation\SelfEvaluationInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Returns responses for themes admin routes.
 */
class SelfEvaluationThemesQuestionsController extends SelfEvaluationListController {

  use ActiveTrailTrait;

  /**
   * Constructs a new SelfEvaluationAdminController.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   Entity Type manager service.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityTypeManager = $entity_type_manager;

  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): self {
    return new static(
      $container->get('entity_type.manager'),
    );
  }

  /**
   * Return Themes List for a self evaluation.
   *
   * @param \Drupal\self_evaluation\SelfEvaluationInterface $self_evaluation
   *   The current Self Evaluation Entity.
   *
   * @return array
   *   Array return.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function buildList(SelfEvaluationInterface $self_evaluation): array {
    $rows = [];
    $list_themes = SelfEvaluationTheme::getThemesFromSelfEvaluation($self_evaluation);

    if (!empty($list_themes)) {
      $themes = $this->entityTypeManager->getStorage('self_evaluation_theme')
        ->loadMultiple($list_themes);

      foreach ($themes as $theme) {
        $rows[] = $this->buildRow($theme);
      }
    }
    $build = $this->render();

    $build['build_active_trail'] = $this->buildActiveTrail($self_evaluation);
    $build['add']['#title'] = $this->t('Add a theme');
    $build['add']['#url'] = Url::fromRoute('entity.self_evaluation_theme.add_form', [], ['query' => ['self_evaluation' => $self_evaluation->id()]]);
    $build['list']['#empty'] = $this->t('There are no theme yet.');
    $build['list']['#rows'] = $rows;
    return $build;
  }

  /**
   * Builds the header row for the entity listing.
   *
   * @return array
   *   A render array structure of header strings.
   *
   * @see \Drupal\Core\Entity\EntityListBuilder::render()
   */
  public function buildHeader(): array {
    $header['title'] = $this->t('Title');
    $header['questions'] = $this->t('Questions');
    $header['thresholds'] = $this->t('Thresholds');
    $header['weight'] = $this->t('Weight');
    $header['status'] = $this->t('Publishing status');
    return $header + parent::buildHeader();
  }

  /**
   * Builds a row for an entity in the entity listing.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity for this row of the list.
   *
   * @return array
   *   A render array structure of fields for this entity.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   *
   * @see \Drupal\Core\Entity\EntityListBuilder::render()
   */
  public function buildRow(EntityInterface $entity): array {
    $row['title']['data'] = $entity->label();
    $row['questions']['data'] = $this->getQuestions($entity);
    $row['thresholds']['data'] = $this->getThresholdsCell($entity);
    $row['weight']['data'] = $entity->weight->value;
    $row['status']['data'] = $entity->isEnabled() ? $this->t('Published') : $this->t('Unpublished');
    return $row + parent::buildRow($entity);
  }

  /**
   * Get data for questions cell for a theme.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   Current Entity Theme.
   *
   * @return array
   *   Data.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  private function getQuestions(EntityInterface $entity): array {
    $list_questions = SelfEvaluationQuestion::getQuestionsFromTheme($entity);
    if (count($list_questions) > 0) {
      return [
        '#type' => 'link',
        '#title' => $this->t("List of questions"),
        '#url' => Url::fromRoute('entity.self_evaluation_theme.questions', ['self_evaluation_theme' => $entity->id()]),
      ];
    }
    return [
      '#type' => 'link',
      '#title' => $this->t("Add a question"),
      '#url' => Url::fromRoute('entity.self_evaluation_question.add_form', [], ['query' => ['self_evaluation_theme' => $entity->id()]]),
    ];
  }

  /**
   * Get data for thresholds cell for a theme.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   Current Entity Theme.
   *
   * @return array
   *   Data.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  private function getThresholdsCell(EntityInterface $entity): array {
    $list_thresholds = SelfEvaluationThemeThreshold::getThresholdsFromTheme($entity);
    if (count($list_thresholds) > 0) {
      return [
        '#type' => 'link',
        '#title' => $this->t("List of thresholds"),
        '#url' => Url::fromRoute('entity.self_evaluation_theme.thresholds', ['self_evaluation_theme' => $entity->id()]),
      ];
    }
    return [
      '#type' => 'link',
      '#title' => $this->t("Add a Threshold"),
      '#url' => Url::fromRoute('entity.self_evaluation_theme_threshold.add_form', [], ['query' => ['self_evaluation_theme' => $entity->id()]]),
    ];
  }

}
