<?php

namespace Drupal\self_evaluation\Entity;

use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Entity\EntityChangedTrait;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\Url;
use Drupal\self_evaluation\SelfEvaluationInterface;
use Drupal\user\UserInterface;

/**
 * Defines the self evaluation entity class.
 *
 * @ContentEntityType(
 *   id = "self_evaluation",
 *   label = @Translation("Self Evaluation"),
 *   label_collection = @Translation("Self Evaluations"),
 *   handlers = {
 *     "view_builder" = "Drupal\Core\Entity\EntityViewBuilder",
 *     "list_builder" = "Drupal\self_evaluation\SelfEvaluationListBuilder",
 *     "views_data" = "Drupal\views\EntityViewsData",
 *     "access" = "Drupal\self_evaluation\SelfEvaluationAccessControlHandler",
 *     "form" = {
 *       "add" = "Drupal\self_evaluation\Form\SelfEvaluationForm",
 *       "edit" = "Drupal\self_evaluation\Form\SelfEvaluationForm",
 *       "delete" = "Drupal\Core\Entity\ContentEntityDeleteForm"
 *     },
 *     "route_provider" = {
 *       "html" = "Drupal\Core\Entity\Routing\AdminHtmlRouteProvider",
 *     }
 *   },
 *   base_table = "self_evaluation",
 *   data_table = "self_evaluation_field_data",
 *   translatable = TRUE,
 *   admin_permission = "administer self evaluation",
 *   entity_keys = {
 *     "id" = "id",
 *     "langcode" = "langcode",
 *     "label" = "title",
 *     "uuid" = "uuid"
 *   },
 *   links = {
 *     "add-form" =
 *   "/admin/self-evaluation/content/self-evaluation/add",
 *     "canonical" =
 *   "/self_evaluation/{self_evaluation}",
 *     "edit-form" =
 *   "/admin/self-evaluation/content/self-evaluation/{self_evaluation}/edit",
 *     "delete-form" =
 *   "/admin/self-evaluation/content/self-evaluation/{self_evaluation}/delete",
 *     "collection" =
 *   "/admin/self-evaluation/content/self-evaluation"
 *   },
 *   field_ui_base_route = "entity.self_evaluation.settings"
 * )
 */
class SelfEvaluation extends ContentEntityBase implements SelfEvaluationInterface {

  use EntityChangedTrait;

  /**
   * {@inheritdoc}
   *
   * When a new self evaluation entity is created, set the uid entity reference
   * to the current user as the creator of the entity.
   */
  public static function preCreate(EntityStorageInterface $storage_controller, array &$values) {
    parent::preCreate($storage_controller, $values);
    $values += ['uid' => \Drupal::currentUser()->id()];
  }

  /**
   * {@inheritdoc}
   */
  public function getTitle() {
    return $this->get('title')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setTitle($title) {
    $this->set('title', $title);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function isEnabled() {
    return (bool) $this->get('status')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setStatus($status) {
    $this->set('status', $status);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getCreatedTime() {
    return $this->get('created')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setCreatedTime($timestamp) {
    $this->set('created', $timestamp);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getOwner() {
    return $this->get('uid')->entity;
  }

  /**
   * {@inheritdoc}
   */
  public function getOwnerId() {
    return $this->get('uid')->target_id;
  }

  /**
   * {@inheritdoc}
   */
  public function setOwnerId($uid) {
    $this->set('uid', $uid);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function setOwner(UserInterface $account) {
    $this->set('uid', $account->id());
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function isPublishable(): bool {
    return (bool) $this->get('publishable_results')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function isResearchable(): bool {
    return (bool) $this->get('publishable_results')->value;
  }

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {

    $fields = parent::baseFieldDefinitions($entity_type);

    $fields['title'] = BaseFieldDefinition::create('string')
      ->setTranslatable(TRUE)
      ->setLabel(t('Title'))
      ->setDescription(t('The title of the self evaluation entity.'))
      ->setRequired(TRUE)
      ->setSetting('max_length', 255)
      ->setDisplayOptions('form', [
        'type' => 'string_textfield',
        'weight' => -5,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayOptions('view', [
        'label' => 'hidden',
        'type' => 'string',
        'weight' => -5,
      ])
      ->setDisplayConfigurable('view', TRUE);

    $fields['description'] = BaseFieldDefinition::create('text_long')
      ->setTranslatable(TRUE)
      ->setLabel(t('Description'))
      ->setDescription(t('A description of the self evaluation.'))
      ->setDisplayOptions('form', [
        'type' => 'text_textarea',
        'weight' => -4,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayOptions('view', [
        'type' => 'text_default',
        'label' => 'above',
        'weight' => -4,
      ])
      ->setDisplayConfigurable('view', TRUE);

    $fields['image'] = BaseFieldDefinition::create('image')
      ->setLabel(t('Image'))
      ->setDescription(t('An illustration for the self evaluation.'))
      ->setDisplayOptions('form', [
        'type' => 'image_image',
        'settings' => [
          'display_label' => FALSE,
          'progress_indicator' => 'throbber',
          'preview_image_style' => 'thumbnail',
        ],
        'weight' => -3,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayOptions('view', [
        'type' => 'image',
        'weight' => -3,
        'label' => 'above',
        'settings' => [
          'image_style' => 'thumbnail',
        ],
      ])
      ->setDisplayConfigurable('view', TRUE)
      ->setReadOnly(TRUE);

    $note_type = [
      'absolute' => t('Absolute'),
      'base_10' => t('Base 10'),
      'base_5' => t('Base 5'),
      'percent' => t('Percent'),
    ];
    $fields['note_type'] = BaseFieldDefinition::create('list_string')
      ->setLabel(t('Note type'))
      ->setDescription(t(''))
      ->setRequired(TRUE)
      ->setSetting('max_length', 128)
      ->setSetting('allowed_values', $note_type)
      ->setDisplayOptions('form', [
        'type' => 'options_select',
        'weight' => -2,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayOptions('view', [
        'label' => 'above',
        'type' => 'string',
        'weight' => -2,
      ])
      ->setDisplayConfigurable('view', TRUE);

    $chart_type = [
      'spider' => t('Spider'),
      'bar' => t('Bar'),
    ];
    $fields['chart_type'] = BaseFieldDefinition::create('list_string')
      ->setLabel(t('Chart type'))
      ->setDescription(t(''))
      ->setSetting('max_length', 128)
      ->setSetting('allowed_values', $chart_type)
      ->setCardinality(BaseFieldDefinition::CARDINALITY_UNLIMITED)
      ->setDisplayOptions('form', [
        'type' => 'options_select',
        'weight' => -1,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayOptions('view', [
        'label' => 'above',
        'type' => 'string',
        'weight' => -1,
      ])
      ->setDisplayConfigurable('view', TRUE);

    $form_steps_mode = [
      'theme' => t('Theme'),
      'question' => t('Question'),
    ];
    $fields['form_steps_mode'] = BaseFieldDefinition::create('list_string')
      ->setRequired(TRUE)
      ->setLabel(t('Form steps mode'))
      ->setDescription(t(''))
      ->setSetting('max_length', 128)
      ->setSetting('allowed_values', $form_steps_mode)
      ->setDisplayOptions('form', [
        'type' => 'options_select',
        'weight' => 0,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayOptions('view', [
        'label' => 'above',
        'type' => 'string',
        'weight' => 0,
      ])
      ->setDisplayConfigurable('view', TRUE);

    $fields['self_evaluation_groupment_types'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Groupment types'))
      ->setDescription(t(''))
      ->setCardinality(BaseFieldDefinition::CARDINALITY_UNLIMITED)
      ->setSetting('target_type', 'self_evaluation_groupment_type')
      ->setDisplayOptions('form', [
        'type' => 'entity_reference_autocomplete',
        'settings' => [
          'match_operator' => 'CONTAINS',
          'size' => 60,
          'placeholder' => '',
        ],
        'weight' => 1,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayOptions('view', [
        'label' => 'above',
        'weight' => 1,
      ])
      ->setDisplayConfigurable('view', TRUE);

    $fields['self_evaluation_groupment_types_comparables'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Groupment types comparables'))
      ->setDescription(t(''))
      ->setCardinality(BaseFieldDefinition::CARDINALITY_UNLIMITED)
      ->setSetting('target_type', 'self_evaluation_groupment_type')
      ->setDisplayOptions('form', [
        'type' => 'entity_reference_autocomplete',
        'settings' => [
          'match_operator' => 'CONTAINS',
          'size' => 60,
          'placeholder' => '',
        ],
        'weight' => 0,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayOptions('view', [
        'label' => 'above',
        'weight' => 0,
      ])
      ->setDisplayConfigurable('view', TRUE);

    $fields['start_button_label'] = BaseFieldDefinition::create('string')
      ->setTranslatable(TRUE)
      ->setLabel(t('Start button label'))
      ->setDescription(t(''))
      ->setRequired(TRUE)
      ->setSetting('max_length', 255)
      ->setDefaultValue(t('Start evaluation'))
      ->setDisplayOptions('form', [
        'type' => 'string_textfield',
        'weight' => 3,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayOptions('view', [
        'label' => 'above',
        'type' => 'string',
        'weight' => 3,
      ])
      ->setDisplayConfigurable('view', TRUE);

    $fields['shareable_results'] = BaseFieldDefinition::create('boolean')
      ->setLabel(t('Shareable results'))
      ->setDescription(t(''))
      ->setDefaultValue(FALSE)
      ->setDisplayOptions('form', [
        'settings' => ['display_label' => TRUE],
        'type' => 'boolean_checkbox',
        'weight' => 4,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayOptions('view', [
        'label' => 'above',
        'type' => 'boolean',
        'weight' => 4,
      ])
      ->setDisplayConfigurable('view', TRUE);

    $fields['publishable_results'] = BaseFieldDefinition::create('boolean')
      ->setLabel(t('Publishable results'))
      ->setDescription(t(''))
      ->setDefaultValue(FALSE)
      ->setDisplayOptions('form', [
        'settings' => ['display_label' => TRUE],
        'type' => 'boolean_checkbox',
        'weight' => 5,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayOptions('view', [
        'label' => 'above',
        'type' => 'boolean',
        'weight' => 5,
      ])
      ->setDisplayConfigurable('view', TRUE);

    $fields['enable_reminder'] = BaseFieldDefinition::create('boolean')
      ->setLabel(t('Enable reminder'))
      ->setDescription(t(''))
      ->setDefaultValue(FALSE)
      ->setDisplayOptions('form', [
        'settings' => ['display_label' => TRUE],
        'type' => 'boolean_checkbox',
        'weight' => 6,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayOptions('view', [
        'label' => 'above',
        'type' => 'boolean',
        'weight' => 6,
      ])
      ->setDisplayConfigurable('view', TRUE);

    $fields['reminder_time_limits'] = BaseFieldDefinition::create('integer')
      ->setLabel(t('Reminder time limits'))
      ->setDescription(t(''))
      ->setSetting('unsigned', TRUE)
      ->setSetting('size', 'big')
      ->setCardinality(BaseFieldDefinition::CARDINALITY_UNLIMITED)
      ->setDisplayOptions('form', [
        'type' => 'number',
        'settings' => [
          'display_label' => TRUE,
          'size' => 5,
        ],
        'weight' => 7,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayOptions('view', [
        'label' => 'above',
        'type' => 'number_integer',
        'weight' => 7,
      ])
      ->setDisplayConfigurable('view', TRUE);

    $fields['reminder_email_template'] = BaseFieldDefinition::create('text_long')
      ->setLabel(t('Reminder Email Template'))
      ->setDescription(t(''))
      ->setDisplayOptions('form', [
        'type' => 'text_textarea',
        'weight' => 8,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayOptions('view', [
        'type' => 'text_default',
        'label' => 'above',
        'weight' => 8,
      ])
      ->setDisplayConfigurable('view', TRUE);

    $fields['uid'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('User'))
      ->setDescription(t('The user ID of the self evaluation author.'))
      ->setSetting('target_type', 'user')
      ->setDisplayOptions('form', [
        'type' => 'entity_reference_autocomplete',
        'settings' => [
          'match_operator' => 'CONTAINS',
          'size' => 11,
          'placeholder' => '',
        ],
        'weight' => 11,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayOptions('view', [
        'region' => 'hidden',
      ])
      ->setDisplayConfigurable('view', TRUE);

    $fields['created'] = BaseFieldDefinition::create('created')
      ->setLabel(t('Creation Date'))
      ->setDescription(t('The time that the self evaluation was created.'))
      ->setDisplayOptions('form', [
        'type' => 'datetime_timestamp',
        'weight' => 12,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayOptions('view', [
        'region' => 'hidden',
      ])
      ->setDisplayConfigurable('view', TRUE);

    $fields['changed'] = BaseFieldDefinition::create('changed')
      ->setLabel(t('Modification Date'))
      ->setDescription(t('The time that the self evaluation was last edited.'));

    $fields['status'] = BaseFieldDefinition::create('boolean')
      ->setLabel(t('Publishing status'))
      ->setDescription(t('A boolean indicating whether the self evaluation is enabled.'))
      ->setDefaultValue(TRUE)
      ->setSetting('on_label', 'Enabled')
      ->setDisplayOptions('form', [
        'type' => 'boolean_checkbox',
        'settings' => [
          'display_label' => FALSE,
        ],
        'weight' => 13,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayOptions('view', [
        'region' => 'hidden',
      ])
      ->setDisplayConfigurable('view', TRUE);

    $fields['path'] = BaseFieldDefinition::create('path')
      ->setLabel(t('URL alias'))
      ->setTranslatable(TRUE)
      ->setDisplayOptions('form', [
        'type' => 'path',
        'weight' => 14,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setComputed(TRUE);

    $langcode = $fields[$entity_type->getKey('langcode')];
    $form = $langcode->getDisplayOptions('form');
    $form['weight'] = 14;
    $langcode->getFieldStorageDefinition()->setDisplayOptions('form', $form);

    return $fields;
  }

  /**
   * {@inheritdoc}
   */
  public static function getExtraFieldDefinitions(): array {
    return [
      'display' => [
        'self_evaluation_title' => [
          'label' => t('SelfEvaluation : Title'),
          'visible' => FALSE,
          'method' => 'buildTitle',
        ],
        'self_evaluation_start_button' => [
          'label' => t('SelfEvaluation : Start button'),
          'visible' => FALSE,
          'method' => 'buildStartButton',
        ],
      ],
    ];
  }

  /**
   * Display the "Title".
   *
   * @return array
   *   Render array.
   */
  public function buildTitle(): array {
    return [
      '#type' => 'html_tag',
      '#tag' => 'h1',
      '#value' => $this->label(),
      '#attributes' => ['class' => ['title']],
    ];
  }

  /**
   * Display the "Start button".
   *
   * @return array
   *   Render array.
   *
   * @throws \Drupal\Core\Entity\EntityMalformedException
   */
  public function buildStartButton(): array {
    $current_user = \Drupal::currentUser();

    $text = $current_user->isAnonymous()
      ? t('Sign in')
      : $this->start_button_label->value;
    $url = $current_user->isAnonymous()
      ? Url::fromRoute(
        'user.login',
        [],
        [
          'query' => [
            'destination' => $this->toUrl()->toString(),
          ],
        ]
      )
      : Url::fromRoute(
        'entity.self_evaluation.form',
        [
          'self_evaluation' => $this->id(),
        ]
      );

    return [
      '#type' => 'link',
      '#title' => $text,
      '#url' => $url,
      '#attributes' => ['class' => ['button', 'connect']],
    ];
  }

}
