<?php

namespace Drupal\self_evaluation\Entity;

use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\self_evaluation\SelfEvaluationQuestionChoiceInterface;

/**
 * Defines the self evaluation question choice entity class.
 *
 * @ContentEntityType(
 *   id = "self_evaluation_question_choice",
 *   label = @Translation("Question Choice"),
 *   label_collection = @Translation("Question Choices"),
 *   handlers = {
 *     "view_builder" = "Drupal\Core\Entity\EntityViewBuilder",
 *     "list_builder" =
 *   "Drupal\self_evaluation\SelfEvaluationQuestionChoiceListBuilder",
 *     "views_data" = "Drupal\views\EntityViewsData",
 *     "access" =
 *   "Drupal\self_evaluation\SelfEvaluationQuestionChoiceAccessControlHandler",
 *     "form" = {
 *       "add" =
 *   "Drupal\self_evaluation\Form\SelfEvaluationQuestionChoiceForm",
 *       "edit" =
 *   "Drupal\self_evaluation\Form\SelfEvaluationQuestionChoiceForm",
 *       "delete" = "Drupal\Core\Entity\ContentEntityDeleteForm"
 *     },
 *     "route_provider" = {
 *       "html" = "Drupal\Core\Entity\Routing\AdminHtmlRouteProvider",
 *     }
 *   },
 *   base_table = "self_evaluation_question_choice",
 *   data_table = "self_evaluation_question_choice_field_data",
 *   translatable = TRUE,
 *   admin_permission = "administer self evaluation question choice",
 *   entity_keys = {
 *     "id" = "id",
 *     "langcode" = "langcode",
 *     "label" = "title",
 *     "uuid" = "uuid"
 *   },
 *   links = {
 *     "add-form" =
 *   "/admin/self-evaluation/content/self-evaluation-question-choice/add",
 *     "canonical" =
 *   "/self_evaluation_question_choice/{self_evaluation_question_choice}",
 *     "edit-form" =
 *   "/admin/self-evaluation/content/self-evaluation-question-choice/{self_evaluation_question_choice}/edit",
 *     "delete-form" =
 *   "/admin/self-evaluation/content/self-evaluation-question-choice/{self_evaluation_question_choice}/delete",
 *     "collection" =
 *   "/admin/self-evaluation/content/self-evaluation-question-choice"
 *   },
 *   field_ui_base_route = "entity.self_evaluation_question_choice.settings"
 * )
 */
class SelfEvaluationQuestionChoice extends ContentEntityBase implements SelfEvaluationQuestionChoiceInterface {

  /**
   * {@inheritdoc}
   */
  public function getTitle() {
    return $this->get('title')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setTitle($title) {
    $this->set('title', $title);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {

    $fields = parent::baseFieldDefinitions($entity_type);

    $fields['title'] = BaseFieldDefinition::create('text_long')
      ->setTranslatable(TRUE)
      ->setLabel(t('Title'))
      ->setDescription(t('The title of the self evaluation question choice entity.'))
      ->setRequired(TRUE)
      ->setSetting('max_length', 255)
      ->setDisplayOptions('form', [
        'type' => 'text_textarea',
        'weight' => -5,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayOptions('view', [
        'type' => 'text_default',
        'label' => 'above',
        'weight' => -5,
      ])
      ->setDisplayConfigurable('view', TRUE);

    $fields['self_evaluation_question'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Parent Question'))
      ->setDescription(t(''))
      ->setRequired(TRUE)
      ->setSetting('target_type', 'self_evaluation_question')
      ->setDisplayOptions('form', [
        'type' => 'entity_reference_autocomplete',
        'settings' => [
          'match_operator' => 'CONTAINS',
          'size' => 60,
          'placeholder' => '',
        ],
        'weight' => -4,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayOptions('view', [
        'label' => 'above',
        'weight' => -4,
      ])
      ->setDisplayConfigurable('view', TRUE);

    $fields['note'] = BaseFieldDefinition::create('decimal')
      ->setLabel(t('Note'))
      ->setDescription(t(''))
      ->setRequired(TRUE)
      ->setSetting('precision', 10)
      ->setSetting('scale', 2)
      ->setDisplayOptions('view', [
        'label' => 'above',
        'type' => 'number_decimal',
        'weight' => -3,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayOptions('form', [
        'type' => 'number',
        'settings' => [
          'display_label' => TRUE,
          'size' => 5,
        ],
        'weight' => -3,
      ])
      ->setDisplayConfigurable('view', TRUE);

    $fields['weight'] = BaseFieldDefinition::create('integer')
      ->setLabel(t('Weight'))
      ->setDescription(t('The weight of this item.'))
      ->setDefaultValue(0)
      ->setSetting('unsigned', TRUE)
      ->setDisplayOptions('form', [
        'type' => 'number',
        'settings' => [
          'display_label' => TRUE,
          'size' => 3,
        ],
        'weight' => -4,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayOptions('view', [
        'label' => 'above',
        'type' => 'number_integer',
        'weight' => -4,
      ])
      ->setDisplayConfigurable('view', TRUE);

    return $fields;
  }

  /**
   * Get Choices for a specific question.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The current theme.
   *
   * @return array|int
   *   Array of choices.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public static function getChoicesForQuestion(EntityInterface $entity) {
    return \Drupal::entityTypeManager()
      ->getStorage('self_evaluation_question_choice')
      ->getQuery()
      ->condition('self_evaluation_question', $entity->id())
      ->execute();
  }

  /**
   * Get Choices for a specific question.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The current theme.
   *
   * @return array|int
   *   Array of choices.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public static function getChoicesFromQuestion(EntityInterface $entity) {
    return \Drupal::entityTypeManager()
      ->getStorage('self_evaluation_question_choice')
      ->getQuery()
      ->condition('self_evaluation_question', $entity->id())
      ->sort('weight', 'ASC')
      ->execute();
  }

}
